/**
 * Author: Dylan Senderling
 * Description: A simple animation that responds to mouse clicks and key presses. If the user
 *              presses the 'd' key, the canvas will show daytime with a sun and moving clouds.
 *              If the user presses the 'n' key, the canvas will show nighttime with a crescent
 *              moon and twinkling stars. If the user clicks the mouse, the stars will not 
 *              twinkle and the clouds will not move.
 */

void setup() {
  size(300, 300);
  background(10, 100, 255);  // Blue sky
  noStroke();
}

boolean daytime = true;   // Default to day
boolean motion  = true;   // Default to animate

int offset  = 0;          // Starting offset position
int twinkle = 255;        // Starting color of stars

void draw() {
  sky(daytime);             // Draws background color
  heavenly_body(daytime);   // Draws either sun or moon
  clouds(daytime);          // Draws the clouds
  stars(daytime);           // Draws the stars
  ground(daytime);          // Draws the hill
  update(daytime);          // Changes offset/twinkle values to create animation
} 

/**
 * This function will draw multiple clouds by making
 * many calls to cloud(x, y) (refer below). It will 
 * only draw if the parameter, day, is true. Day should
 * be true if the canvas is in daytime mode and false
 * if in nighttime mode.
 */
void clouds(boolean day) {
  if(day) {
    cloud(50 + offset, 10);
    cloud(150 + offset, 200);
    cloud(350 + offset, 100);
    cloud(450 + offset, 150);
    cloud(550 + offset, 10);
    cloud(650 + offset, 200);
    cloud(850 + offset, 100);
  }
}

/**
 * This function will draw multiple stars by making
 * many calls to star(x, y) (refer below). It will 
 * only draw if the parameter, day, is false. Day should
 * be true if the canvas is in daytime mode and false
 * if in nighttime mode.
 */
void stars(boolean day) {
  if(!day) {
    star(10, 25);
    star(100, 75);
    star(50, 125);
    star(150, 175);
    star(190, 50);
  }
}

/**
 * This function will draw a green hill, a circle that
 * only partially shows on the screen. It will be a dark
 * green if day is false, bright green if true. Day should
 * be true if the canvas is in daytime mode and false if 
 * in nighttime mode.
 */
void ground(boolean day) {
  fill(10, 255, 10);       // bright green
  if(!day) {               // if night (not day...)
    fill(10, 150, 10);     // darker green
  }
  ellipse(150, 400, 400, 400);
}

/**
 * This function will draw the background. Uses blue if
 * day is true, black otherwise. Day should be true if 
 * the canvas is in daytime mode and false if in 
 * nighttime mode.
 */
void sky(boolean day) {
  background(0);              // night sky
  if(day) {                   // if daytime...
    background(10, 100, 255); // blue sky
  }
}

/**
 * This function will draw a sun if day is true and a 
 * crescent moon if day is false. This works by changing 
 * the fill value of the ellipse as well as drawing a 
 * black circle over the moon if day is false. Day 
 * should be true if the canvas is in daytime mode and 
 * false if in nighttime mode.
 */
void heavenly_body(boolean day) {
  for(int i = 100; i > 0; i-=5) {   // This for loop creates a cool glowing effect
    fill(255, i);                   // It draws stacked ellipses but the transparency
    if(day) {                       // gets less and less as it draws toward the center
      fill(255, 255, 10, i + 50);   // of the planet
    }
    ellipse(225, 75, i, i);
  }
  if(!day) {                     // if night (not day...)
    fill(0);
    ellipse(200, 50, 100, 100);  // cover up part of the moon!
  }
}

/**
 * This function will draw a cloud by making many circles
 * and overlapping them given the x and y position, which
 * will denote the center of the largest circle in the 
 * cloud. Color is white with a bit of transparency.
 */
void cloud(int x, int y) {
  fill(255, 250);
  ellipse(x, y, 50, 50);
  ellipse(x, y + 20, 40, 40);
  ellipse(x - 20, y + 10, 40, 40);
  ellipse(x + 20, y + 10, 40, 40);
}

/**
 * This function will draw a star given the x and y 
 * positions for that star. It is simply a tiny circle,
 * width and height of 3 and center at x, y. Color is
 * whatever twinkle is, some white/grey color.
 */
void star(int x, int y) {
  fill(twinkle);
  ellipse(x, y, 3, 3);
}

/**
 * Event handler for the keys, toggles daytime. Sets 
 * daytime true if 'd' is pressed, false if 'n' is
 * pressed. No other events are detected from keys.
 */
void keyPressed() {
  if(key == 'd') {
    daytime = true;
  } else if(key == 'n') {
    daytime = false;
  }
}

/**
 * Toggles the animation with a mouse click
 */
void mousePressed() {
  motion = !motion;   // This will switch the boolean! If true, now false, and vice versa
}

/**
 * This function will update the offset and twinkle 
 * values. Will decrement offset to make the clouds move
 * to the right, resetting when it hits -500. This is to
 * create the illusion of nonstop motion. Twinkle changes
 * from 255 to 200 by values of 5, resetting to 255 so as
 * to give the illusion of twinkling. This only happens
 * if motion is true, and clouds only move if day is true
 * and stars only twinkle if day is false. Day should
 * be true if the canvas is in daytime mode and false
 * if in nighttime mode.
 **/
void update(boolean day) {
  if(motion) {
    if(day) {
      offset -= 1;
      if(offset < -500) {
        offset = 0;        // reset
      }
    } else {
      if(twinkle > 200) {
        twinkle -= 5;
      } else {
        twinkle = 255;   // reset
      }
    }
  }
}